from functools import partial
from time import sleep

import click

DELAY = 10


def get_instance_configuration(config):
    instance = partial(config.get, "INSTANCE")

    return {
        "InstanceCount": 1,
        "Type": "one-time",
        "InstanceInterruptionBehavior": "terminate",
        "LaunchSpecification": {
            "SecurityGroups": [instance("security_group")],
            "ImageId": instance("ami"),
            "InstanceType": instance("type"),
            "KeyName": instance("key_pair"),
        },
        "SpotPrice": instance("spot_price"),
    }


def wait_until_instance_ready(client, instance_id):
    response = client.describe_instance_status(InstanceIds=[instance_id])
    status = response["InstanceStatuses"][0]["InstanceState"]["Name"]
    while status == "pending":
        response = client.describe_instance_status(InstanceIds=[instance_id])
        status = response["InstanceStatuses"][0]["InstanceState"]["Name"]
        sleep(1)
    return response["InstanceStatuses"][0]["InstanceState"]["Name"]


def request_instance(client, config):

    instance_config = get_instance_configuration(config)
    request_response = client.request_spot_instances(**instance_config)
    current_request = request_response["SpotInstanceRequests"][0]
    request_id = current_request["SpotInstanceRequestId"]
    click.echo(f"Spot request {request_id} created, status: {current_request['State']}")
    spot_request = wait_for_spot_request(client, request_id)

    if spot_request["State"] == "active":
        instance = get_instance_from(client, spot_request)

        client.create_tags(
            Resources=[spot_request["InstanceId"]],
            Tags=[{"Key": "CreatedBy", "Value": "SoftSpot"}],
        )

        instance_id = instance["InstanceId"]
        public_ip = get_public_ip(instance)
        click.echo(f"Instance {instance_id} started, IP: {public_ip}")
        instance_status = wait_until_instance_ready(client, instance_id)
        if instance_status == "running":
            if config.has_section("VOLUME"):
                attach_device(client, instance_id, config)
            click.echo(
                click.style(
                    f"Done! the IP of the image is {public_ip}", bg="blue", fg="white"
                )
            )
        else:
            click.echo(
                f"The instance created changed to a non-running state: {instance_status}"
            )
    else:
        click.echo(f"The request failed, status {spot_request['State']}")


def get_instance_from(client, spot_request):
    instances = client.describe_instances(InstanceIds=[spot_request["InstanceId"]])
    return instances["Reservations"][0]["Instances"][0]


def wait_for_spot_request(client, spot_request_id):
    response = client.describe_spot_instance_requests(
        SpotInstanceRequestIds=[spot_request_id]
    )
    spot_request = response["SpotInstanceRequests"][0]
    while spot_request["State"] == "open":

        response = client.describe_spot_instance_requests(
            SpotInstanceRequestIds=[spot_request_id]
        )
        spot_request = response["SpotInstanceRequests"][0]

        click.echo("Waiting...")
        sleep(DELAY)
    return spot_request


def attach_device(client, instance_id, config):
    volume_id = config.get("VOLUME", "id")
    device = config.get("VOLUME", "device")
    click.echo(f"Will attach the volume {volume_id} to {instance_id} at {device}")
    attachment_result = client.attach_volume(
        VolumeId=volume_id, InstanceId=instance_id, Device=device
    )
    return attachment_result


def get_public_ip(instance):
    return instance["NetworkInterfaces"][0]["Association"]["PublicIp"]



def get_price_request_configuration(config):
    instance_section = config["INSTANCE"]
    price_request_configuration = {"InstanceTypes": [instance_section["type"]]}
    availability_zone = instance_section.get("availability_zone", None)
    product_description = instance_section.get("product_description", None)
    if availability_zone:
        price_request_configuration["AvailabilityZone"] = availability_zone
    if product_description:
        price_request_configuration["ProductDescriptions"] = [product_description]
    return price_request_configuration


def get_prices(client, instance_configuration, start_time, end_time):
    conf = get_price_request_configuration(instance_configuration)
    conf["EndTime"] = end_time
    conf["StartTime"] = start_time
    response = client.describe_spot_price_history(**conf)

    headers = [
        "Timestamp",
        "SpotPrice",
        "AvailabilityZone",
        "InstanceType",
        "ProductDescription",
    ]
    spot_price_history = [
        [p[key] for key in headers] for p in response["SpotPriceHistory"]
    ]

    spot_price_history = sorted(spot_price_history, key=lambda prc: prc[0])
    return headers, spot_price_history



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['soft_spot', 'soft_spot.implementations']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.9,<2.0', 'click>=7.0,<8.0', 'tabulate>=0.8.5,<0.9.0']

entry_points = \
{'console_scripts': ['sspot = soft_spot.__main__:cli']}

setup_kwargs = {
    'name': 'soft-spot',
    'version': '0.2.1',
    'description': 'Move to a land of Spot AWS instances',
    'long_description': '# soft-spot\n\n[![Build Status](https://dev.azure.com/messier-16/soft-spot/_apis/build/status/fferegrino.soft-spot?branchName=master)](https://dev.azure.com/messier-16/soft-spot/_build/latest?definitionId=1&branchName=master) [![PyPI version](https://badge.fury.io/py/soft-spot.svg)](https://pypi.org/project/soft-spot/)\n\nDo you have a soft spot for cheap cloud computing (**a.k.a. AWS Spot instances**)? Me too, no shame on that.\n\nHowever, what is a shame is having to go through that clunky UI and click here and there to get one; `soft-spot` makes it dead easy to launch an instance:\n\n## How?\nJust define a file with the specifications of the machine you want to launch:  \n\n```ini\n[INSTANCE]\nami = ami-06f2f779464715dc5\ntype = t2.micro\nsecurity_group = wizard-launch\nkey_pair = a_secret_key\nspot_price = 0.0035\nproduct_description = Linux/UNIX\navailability_zone = eu-west-2b\n\n[ACCOUNT]\nuser = unbuntu\n\n[VOLUME]\nid = vol-volume123\ndevice = /dev/sda2\nmount_point = /data/\n```\n\nThen just execute the `sspot request` command:  \n\n```bash\nsspot request <<instance_config_file>>\n```\n\n### Credentials  \nThis script uses `boto3` so I strongly recommend heading over to [its documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html) to learn more.  \n\nAlternatively, you could create a tiny configuration file like this:  \n\n```ini\n[DEFAULT]\naws_access_key_id = an_acces_key\naws_secret_access_key = a_secret_key\nregion_name = us-west-1\n```\n\nAnd then pass it on to the `spot` command:\n\n```bash\nsspot -a ~/aws_credentials.txt request <<instance_config_file>> \n```\n',
    'author': 'Antonio Feregrino',
    'author_email': 'antonio.feregrino@gmail.com',
    'url': 'https://github.com/fferegrino/soft-spot',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import configparser


def get_account_info(account_info_file):
    if account_info_file:
        config = configparser.ConfigParser()
        config.read(account_info_file)
        return config["DEFAULT"]
    return {}



import configparser
from datetime import datetime, timedelta

import boto3
import click
from tabulate import tabulate

from soft_spot.configuration import get_account_info
from soft_spot.implementations.price import get_prices
from soft_spot.implementations.request import request_instance


def get_client(account_info):
    return boto3.client("ec2", **account_info)


@click.group()
@click.option(
    "--account_info_file",
    "-a",
    default=None,
    type=click.Path(exists=True, dir_okay=False),
)
@click.pass_context
def cli(context, account_info_file):
    context.ensure_object(dict)
    context.obj["client"] = get_client(get_account_info(account_info_file))


@cli.command()
@click.pass_context
@click.argument("instance_file", type=click.Path(exists=True, dir_okay=False))
def request(context, instance_file):
    click.echo(f"Requesting from: {instance_file}")

    instance_configuration = configparser.ConfigParser()
    instance_configuration.read(instance_file)

    request_instance(context.obj["client"], instance_configuration)


@cli.command()
@click.pass_context
@click.argument("instance_file", type=click.Path(exists=True, dir_okay=False))
@click.option("--start-time", type=click.DateTime(), default=None)
@click.option("--end-time", type=click.DateTime(), default=None)
def price(context, instance_file, start_time, end_time):
    click.echo(f"Requesting from: {instance_file}")

    instance_configuration = configparser.ConfigParser()
    instance_configuration.read(instance_file)

    if end_time is None:
        end_time = datetime.now()
    if start_time is None:
        start_time = end_time - timedelta(days=1)

    headers, prices = get_prices(
        context.obj["client"],
        instance_configuration,
        end_time=end_time,
        start_time=start_time,
    )
    click.echo(tabulate(prices, headers=headers))


if __name__ == "__main__":
    # pylint: disable=E1120
    cli()
